library(tidyverse)
library(parallel)


mutations = read_tsv('../tmp/mutations.txt', comment = '#') %>%
  mutate(
    `hugo_symbol` = factor(`hugo_symbol`),
    `sample_id` = factor(`sample_id`)
  ) %>%
  distinct(`sample_id`, `hugo_symbol`, .keep_all = T)
variants = read_tsv('../tmp/variants.txt', comment = '#')


permutation_test = function(data, test_sample_ids, seed = 123, m = 1000000, verbose = F) {
  set.seed(seed)
  
  samples = data %>% pull(`sample_id`) %>% unique()
  test_samples = data %>% filter(`sample_id` %in% test_sample_ids) %>% pull(`sample_id`) %>% unique()
  
  stat = data %>%
    filter(`sample_id` %in% test_samples) %>%
    count(`hugo_symbol`, name = 'observed_frequency', .drop = F) %>%
    mutate(
      `observed_relative_frequency` = `observed_frequency` / length(test_samples),
      `expected_frequency` = 0,
      `left` = 0,
      `right` = 0
    )
  
  for (i in 1:m) {
    counts = data %>%
      filter(`sample_id` %in% sample(samples, length(test_samples))) %>%
      count(`hugo_symbol`, .drop = F) %>%
      pull(`n`)
    
    stat = stat %>%
      mutate(
        `expected_frequency` = `expected_frequency` + counts,
        `left` = `left` + (counts < `observed_frequency`),
        `right` = `right` + (counts > `observed_frequency`)
      )
    
    if (verbose & i %% 100 == 0) message(sprintf('Completed %s of %s permutations', i, m))
  }
  
  stat = stat %>%
    mutate(
      `expected_frequency` = `expected_frequency` / i,
      `expected_relative_frequency` = `expected_frequency` / length(test_samples),
      `absolute_relative_frequency_difference` = abs(`observed_relative_frequency` - `expected_relative_frequency`),
      `p_left` = `left` / i,
      `p_right` = `right` / i
    ) %>%
    arrange(-`absolute_relative_frequency_difference`)
  
  return(stat)
}


braf_sample_ids = mutations %>% left_join(variants, by = c('hgvsp_short')) %>% filter(count >= 5) %>% pull(`sample_id`)
braf_class_1_sample_ids = mutations %>% left_join(variants, by = c('hgvsp_short')) %>% filter(count >= 5, class == '1') %>% pull(`sample_id`)
braf_class_2_sample_ids = mutations %>% left_join(variants, by = c('hgvsp_short')) %>% filter(count >= 5, class == '2') %>% pull(`sample_id`)
braf_class_3_sample_ids = mutations %>% left_join(variants, by = c('hgvsp_short')) %>% filter(count >= 5, class == '3') %>% pull(`sample_id`)


n_cores = detectCores(logical = FALSE)
cl = makeCluster(10)
clusterEvalQ(cl, {
  library(tidyverse)
})
clusterExport(cl, c('permutation_test'))

stats = parLapply(
  cl, 
  list(
    list(
      data = mutations %>% filter(`cancer_type` == 'Non-Small Cell Lung Cancer'),
      test_sample_ids = braf_sample_ids
    ),
    list(
      data = mutations %>% filter(`cancer_type` == 'Melanoma'),
      test_sample_ids = braf_sample_ids
    ),
    list(
      data = mutations %>% filter(`cancer_type` == 'Thyroid Cancer'),
      test_sample_ids = braf_sample_ids
    ),
    list(
      data = mutations %>% filter(`cancer_type` == 'Colorectal Cancer'),
      test_sample_ids = braf_sample_ids
    ),
    list(
      data = mutations %>% filter(`cancer_type` == 'Non-Small Cell Lung Cancer'),
      test_sample_ids = braf_class_1_sample_ids
    ),
    list(
      data = mutations %>% filter(`cancer_type` == 'Non-Small Cell Lung Cancer'),
      test_sample_ids = braf_class_2_sample_ids
    ),
    list(
      data = mutations %>% filter(`cancer_type` == 'Non-Small Cell Lung Cancer'),
      test_sample_ids = braf_class_3_sample_ids
    )
  ), 
  function (x) {
    do.call(permutation_test, x)
  }
)


stats %>%
  map2(
    c(
      '../tmp/comutation_nsclc.txt',
      '../tmp/comutation_melanoma.txt',
      '../tmp/comutation_thyroid.txt',
      '../tmp/comutation_colorectal.txt',
      '../tmp/comutation_nsclc_class_1.txt',
      '../tmp/comutation_nsclc_class_2.txt',
      '../tmp/comutation_nsclc_class_3.txt'
    ),
    write_tsv
  )