#'
#' Figure 1
#'

library(tidyverse)
library(ggpubr)

# Load data
samples = read_tsv('../tmp/samples.txt', comment = '#')
mutations = read_tsv('../tmp/mutations.txt', comment = '#')
classes = read_tsv('../tmp/classes.txt', comment = '#', col_types = cols(.default = 'c'))

# BRAF variants
variants = mutations %>%
  filter(`hugo_symbol` == 'BRAF') %>%
  mutate(`protein_position` = as.numeric(str_match(`hgvsp_short`, 'p.[A-Z]([0-9]*)')[,2])) %>%
  select(`protein_position`, `hgvsp`, `hgvsp_short`, `variant_classification`) %>%
  group_by(`protein_position`) %>%
  mutate(`protein_position_count` = n()) %>%
  ungroup() %>%
  group_by(`protein_position_count`, `protein_position`, `hgvsp`, `hgvsp_short`, `variant_classification`) %>%
  summarize(`count` = n()) %>%
  arrange(`protein_position_count`, -`protein_position`, `count`) %>%
  left_join(classes, by = 'hgvsp_short') %>%
  mutate(
    `class` = ifelse(`class` %in% c('1', '2', '3'), `class`, 'Unknown'),
    `class` = factor(`class`, levels = c('1', '2', '3', 'Unknown'))
    ) %>%
  write_tsv('../tmp/variants.txt')

# Export variants table
variants %>%
  select(`hgvsp`, `hgvsp_short`, `protein_position`, 
          `protein_position_count`, `count`, `class`) %>%
  arrange(-`protein_position_count`, `protein_position`, -`count`) %>%
  writexl::write_xlsx('../variants.xlsx')

sample_count = variants %>% pull(`count`) %>% sum()
allele_count = nrow(variants)

# Create plots
p1 = variants %>%
  mutate(
    bin = cut(
      `count`, 
      breaks = c(1, 2, 5, 10, 100, 1000, 10000),
      labels = c('1', '2-4', '5-9', '10-99', '100-999', '>=1000'),
      right = F)
    ) %>%
  group_by(bin) %>%
  summarize(n = n()) %>%
  ggbarplot(
    x = 'bin',
    y = 'n',
    ylab = 'Count',
    xlab = 'Frequency',
    label = 'n',
    lab.vjust = -0.5,
    color = get_palette(palette = 'npg', 1),
    fill = get_palette(palette = 'npg', 1)
    ) +
  rotate_x_text(angle = 45) +
  scale_y_continuous(limits = c(0, 700), expand = expansion(add = 0))

p2 = variants %>%
  group_by(`variant_classification`) %>%
  summarize(n = n()) %>%
  arrange(-n) %>%
  mutate(`variant_classification` = str_replace_all(`variant_classification`, '_', ' ')) %>%
  ggbarplot(
    x = 'variant_classification',
    y = 'n',
    ylab = 'Count',
    xlab = 'Variant classification',
    label = 'n',
    lab.vjust = -0.5,
    color = get_palette(palette = 'npg', 1),
    fill = get_palette(palette = 'npg', 1)
  ) +
  rotate_x_text(angle = 45) +
  scale_y_continuous(limits = c(0, 1000), expand = expansion(add = 0))  

p3 = variants %>%
  group_by(`class`) %>%
  summarize(`n` = n()) %>%
  mutate(`label` = paste0('Class ', class, '\nn=', n, '\n', round(n / allele_count * 100, 2), '%')) %>%
  ggdonutchart(
    title = 'BRAF variants',
    x = 'n',
    fill = 'class',
    color = 'class',
    label = 'label',
    palette = 'npg'
  ) + rremove('legend')

p4 = variants %>%
  group_by(`class`) %>%
  summarize(`n` = sum(`count`)) %>%
  mutate(`label` = paste0('Class ', class, '\nn=', n, '\n', round(n / sample_count * 100, 2), '%')) %>%
  ggdonutchart(
    title = 'Samples',
    x = 'n',
    fill = 'class',
    color = 'class',
    label = 'label',
    palette = 'npg',
    legend.title = 'Variant class',
  ) + rremove('legend')

p5 = variants %>%
  mutate(`hgvsp_short` = paste0(`hgvsp_short`, ' (', `class`, ')')) %>%
  filter(`count` >= 5) %>%
  ggbarplot(
    x = 'hgvsp_short',
    y = 'count',
    xlab = 'Variant',
    ylab = 'Frequency',
    fill = 'class',
    color = 'class',
    label = 'count',
    lab.hjust = -0.2,
    lab.vjust = 0.5,
    font.label = c(8, 'plain', 'red'),
    palette = 'npg',
    rotate = T,
    legend = 'bottom'
  ) + 
  scale_y_continuous(expand = expansion(add = c(0, 1000))) +
  font('y.text', size = 8)

# Save figure
p1_4 = ggarrange(p1, p3, p4, labels = c('A', 'C', 'D'), ncol = 1, nrow = 4)

p = ggarrange(p1_4, p5, labels = c('A', 'B'), ncol= 2, nrow = 1, widths = c(0.4, 0.6)) + 
  theme(plot.margin = margin(.5, .5, .5, .5, unit = 'inch'))
ggexport(p, filename = '../figure_1.pdf', width = 8.5, height = 11)


