#'
#' Figure 2
#'

library(tidyverse)
library(ggpubr)

samples = read_tsv('../tmp/samples.txt', comment = '#')
mutations = read_tsv('../tmp/mutations.txt', comment = '#')
cancer_types = read_tsv('../tmp/cancer_types.txt', comment = '#')
variants = read_tsv('../tmp/variants.txt', comment = '#') %>%
  filter(count >= 5)

# BRAF tumor types
braf_tumors = mutations %>%
  right_join(variants, by = 'hgvsp_short') %>%
  mutate(`class` = factor(`class`, levels = c('?', '3', '2', '1'))) %>%
  arrange(desc(`class`)) %>%
  distinct(`sample_id`, .keep_all = T) %>%
  group_by(`cancer_type`) %>%
  mutate(`cancer_type_count` = n()) %>%
  group_by(`cancer_type_count`, `cancer_type_detailed`, .add = T) %>%
  summarize(`cancer_type_detailed_count` = n()) %>%
  arrange(`cancer_type_count`, `cancer_type_detailed_count`) %>%
  left_join(
    cancer_types, 
    by = c('cancer_type', 'cancer_type_detailed'), 
    suffix = c('', '_total')
  ) %>%
  mutate(
    `cancer_type_fraction` = `cancer_type_count` / `cancer_type_count_total`,
    `cancer_type_detailed_fraction` = `cancer_type_detailed_count` / `cancer_type_detailed_count_total`
  ) %>%
  ungroup()

# Create plots
p1 = braf_tumors %>%
  distinct(`cancer_type`, .keep_all = T) %>%
  filter(`cancer_type_count` >= 10) %>%
  arrange(`cancer_type_fraction`) %>%
  mutate(
    `cancer_type_fraction` = `cancer_type_fraction` * 100,
    `label` = paste0(round(`cancer_type_fraction`, digits = 2), '% (', `cancer_type_count`, '/', `cancer_type_count_total`, ')')
  ) %>%
  ggbarplot(
    x = 'cancer_type', 
    y = 'cancer_type_fraction',
    xlab = 'Cancer type',
    ylab = 'Relative frequency (%)',
    label = 'label',
    lab.hjust = -0.1,
    lab.vjust = 0.5,
    orientation = 'horiz',
    color = get_palette(palette = 'npg', 1),
    fill = get_palette(palette = 'npg', 1),
    palette = 'npg',
    legend = 'bottom',
    legend.title = 'Cancer type',
  ) +
  scale_y_continuous(limits = c(0, 100), expand = expansion(add = 0))


p2 = braf_tumors %>%
  filter(
    `cancer_type` %in% c('Thyroid Cancer', 'Melanoma', 'Colorectal Cancer', 'Non-Small Cell Lung Cancer'),
    `cancer_type_detailed_count` >= 10
    ) %>%
  arrange(`cancer_type_fraction`, `cancer_type_detailed_fraction`) %>%
  mutate(
    `cancer_type_detailed_fraction` = `cancer_type_detailed_fraction` * 100,
    `Label` = paste0(round(`cancer_type_detailed_fraction`, digits = 2), '% (', `cancer_type_detailed_count`, '/', `cancer_type_detailed_count_total`, ')')
  ) %>%
  ggbarplot(
    x = 'cancer_type_detailed', 
    y = 'cancer_type_detailed_fraction',
    xlab = 'Cancer type detailed',
    ylab = 'Relative frequency (%)',
    label = 'Label',
    lab.hjust = -0.1,
    lab.vjust = 0.5,
    orientation = 'horiz',
    color = 'cancer_type',
    fill = 'cancer_type',
    palette = 'npg',
    legend = 'bottom',
    legend.title = 'Cancer type',
  ) +
  scale_y_continuous(limits = c(0, 100), expand = expansion(add = 0))


# Save figure
p = ggarrange(p1, p2, labels = c('A', 'B'), ncol= 1, heights = c(0.6, 0.45)) + 
  theme(plot.margin = margin(.5, .5, .5, .5, unit = 'inch'))
ggexport(p, filename = '../figure_2.pdf', width = 8.5, height = 11)



cancer_types = mutations %>%
  right_join(variants, by = 'hgvsp_short') %>%
  mutate(`class` = factor(`class`, levels = c('?', '3', '2', '1'))) %>%
  arrange(desc(`class`)) %>%
  distinct(`sample_id`, .keep_all = T) %>%
  group_by(`cancer_type`, `class`) %>%
  summarize(`f` = n()) %>%
  group_by(`cancer_type`) %>%
  mutate(
    `n` = sum(`f`),
    `rf` = `f` / `n`,
    `class_1_rf` = ifelse(any(`class` == '1'), `rf`[`class` == 1], 0)
  ) %>%
  arrange(`class_1_rf`, desc(`class`)) %>%
  filter(
    `cancer_type` %in% (braf_tumors %>% filter(`cancer_type_count` >= 10) %>% pull(`cancer_type`))
    )


p3 = ggbarplot(
  cancer_types,
  x = 'cancer_type', 
  y = 'rf',
  xlab = 'Cancer type',
  ylab = 'Relative frequency',
  color = 'class',
  fill = 'class',
  palette = 'npg',
  legend = 'none'
) +
  geom_text(
    aes(x = `cancer_type`, y = 1, label = `n`),
    data = cancer_types %>% distinct(`cancer_type`, .keep_all = T),
    hjust = 'left',
    nudge_y = 0.01
  ) +
  coord_flip(ylim = c(0, 1), clip = 'off') +
  scale_y_continuous(expand = expansion(add = c(0, .1)))


cancer_types_detailed = mutations %>%
  filter(`cancer_type` %in% c('Thyroid Cancer', 'Melanoma', 'Colorectal Cancer', 'Non-Small Cell Lung Cancer')) %>%
  right_join(variants, by = 'hgvsp_short') %>%
  group_by(`cancer_type`, `cancer_type_detailed`, `class`) %>%
  summarize(`f` = n()) %>%
  group_by(`cancer_type`, `cancer_type_detailed`) %>%
  mutate(
    `n` = sum(`f`),
    `rf` = `f` / `n`,
    `class` = factor(`class`, levels = c('?', '3', '2', '1')),
    `class_1_rf` = ifelse(any(`class` == '1'), `rf`[`class` == 1], 0)
  ) %>%
  group_by(`cancer_type`) %>%
  mutate(
    `m` = sum(`n`)
  ) %>%
  ungroup() %>%
  arrange(`m`, `class_1_rf`, desc(`class`)) %>%
  filter(
    `cancer_type_detailed` %in% (braf_tumors %>% filter(`cancer_type_detailed_count` >= 10) %>% pull(`cancer_type_detailed`))
  )


p4 = ggbarplot(
  cancer_types_detailed,
  x = 'cancer_type_detailed', 
  y = 'rf',
  xlab = 'Cancer type detailed',
  ylab = 'Relative frequency',
  color = 'class',
  fill = 'class',
  legend = 'bottom',
  palette = 'npg'
) +
  geom_text(
    aes(x = `cancer_type_detailed`, y = 1, label = `n`),
    data = cancer_types_detailed %>% distinct(`cancer_type_detailed`, .keep_all = T),
    hjust = 'left',
    nudge_y = 0.01
  ) +
  coord_flip(ylim = c(0, 1), clip = 'off') +
  scale_y_continuous(expand = expansion(add = c(0, .1)))


# Save figure
p = ggarrange(p3, p4, labels = c('A', 'B'), ncol= 1, heights = c(0.6, 0.45)) + 
  theme(plot.margin = margin(.5, .5, .5, .5, unit = 'inch'))
ggexport(p, filename = '../figure_3.pdf', width = 8.5, height = 11)

