#'
#' Figure 5/6
#'

library(tidyverse)
library(ggpubr)


stat_plot = function(file, absolute_relative_frequency_difference_cutoff, fdr_cutoff = 0.001, title, xlim = 1, ylim = 1) {
  read_tsv(file) %>%
    filter(
      `absolute_relative_frequency_difference` >= absolute_relative_frequency_difference_cutoff,
      `hugo_symbol` != 'BRAF'
    ) %>%
    mutate(
      `p` = pmin(`p_left`, `p_right`) * 2,
      `fdr` = p.adjust(`p`, method = 'fdr')
      ) %>%
    filter(`fdr` <= fdr_cutoff) %>%
    ggscatter(
      'expected_relative_frequency',
      'observed_relative_frequency',
      xlab = 'Expected relative frequency',
      ylab = 'Observed relative frequency',
      title = title,
      xlim = c(0, xlim),
      ylim = c(0, ylim),
      label = 'hugo_symbol', 
      repel = T
    ) + 
    geom_abline(intercept = 0, slope = 1, lty = 'dashed')
}

p1 = stat_plot('../tmp/comutation_thyroid.txt', 0.02, 0.001, 'Thyroid cancer', xlim = 0.25, ylim = 0.25)
p2 = stat_plot('../tmp/comutation_melanoma.txt', 0.02, 0.001, 'Melanoma', xlim = 0.25, ylim = 0.25)
p3 = stat_plot('../tmp/comutation_colorectal.txt', 0.06, 0.001, 'Colorectal cancer', xlim = 0.75, ylim = 0.75)
p4 = stat_plot('../tmp/comutation_nsclc.txt', 0.02, 0.001, 'Non-small cell lung cancer', xlim = 0.75, ylim = 0.75)

# Save figure
p = ggarrange(p1, p2, p3, p4, labels = c('A', 'B', 'C', 'D'), ncol= 2, nrow = 2) + 
  theme(plot.margin = margin(.5, .5, 3, .5, unit = 'inch'))
ggexport(p, filename = '../figure_5.pdf', width = 8.5, height = 11)


p5 = stat_plot('../tmp/1m_permutations/comutation_nsclc_class_1.txt', 0.02, 0.001, 'Class 1', xlim = 0.6, ylim = 0.6)
p6 = stat_plot('../tmp/1m_permutations/comutation_nsclc_class_2.txt', 0.02, 0.001, 'Class 2', xlim = 0.6, ylim = 0.6)
p7 = stat_plot('../tmp/1m_permutations/comutation_nsclc_class_3.txt', 0.02, 0.001, 'Class 3', xlim = 0.6, ylim = 0.6)


perm = bind_rows(
  read_tsv('../tmp/1m_permutations/comutation_nsclc_class_1.txt') %>% mutate(`class` = '1'),
  read_tsv('../tmp/1m_permutations/comutation_nsclc_class_2.txt') %>% mutate(`class` = '2'),
  read_tsv('../tmp/1m_permutations/comutation_nsclc_class_3.txt') %>% mutate(`class` = '3')
  ) %>%
  filter(`hugo_symbol` %in% c('KRAS', 'EGFR', 'STK11', 'KEAP1', 'TP53', 'SETD2', 'NRAS', 'ATR', 'AKT1')) %>%
  mutate(`ratio` = log2(`observed_frequency` / `expected_frequency`))

p8 = ggplot(perm, aes(x = `class`, y = `hugo_symbol`, fill = `ratio`)) +
  geom_tile() +
  scale_fill_gradient2(
    midpoint = 0,
    space = "Lab",
    na.value = "grey50",
    guide = "colourbar",
    aesthetics = "fill"
  ) +
  labs(x = 'Variant class', y = 'Gene', fill = 'Ratio') +
  scale_x_discrete(expand = c(0, 0)) +
  scale_y_discrete(expand = c(0, 0)) +
  theme_pubclean() +
  theme(
    legend.position = 'right', 
    legend.direction = 'vertical',
    axis.ticks = element_blank()
    #legend.margin = margin(grid::unit(0, 'cm')),
    #legend.text = element_text(colour=textcol, size=7, face="bold"),
    #legend.key.height = grid::unit(0.8, 'cm'),
    #legend.key.width = grid::unit(0.2, 'cm'),
  )


# Save figure
p = ggarrange(p5, p6, p7, p8, labels = c('A', 'B', 'C', 'D'), ncol= 2, nrow = 2) + 
  theme(plot.margin = margin(.5, .5, 3, .5, unit = 'inch'))
ggexport(p, filename = '../figure_6.pdf', width = 8.5, height = 11)



